@@ -36,6 +36,7 @@ from playwright._impl._helper import (
     Error,
     NameValue,
     TargetClosedError,
+    TimeoutSettings,
     async_readfile,
     async_writefile,
     is_file_payload,
@@ -73,6 +74,8 @@ class APIRequest:
         timeout: float = None,
         storageState: Union[StorageState, str, Path] = None,
         clientCertificates: List[ClientCertificate] = None,
+        failOnStatusCode: bool = None,
+        maxRedirects: int = None,
     ) -> "APIRequestContext":
         params = locals_to_params(locals())
         if "storageState" in params:
@@ -88,8 +91,11 @@ class APIRequest:
         )
         context = cast(
             APIRequestContext,
-            from_channel(await self.playwright._channel.send("newRequest", params)),
+            from_channel(
+                await self.playwright._channel.send("newRequest", None, params)
+            ),
         )
+        context._timeout_settings.set_default_timeout(timeout)
         return context
 
 
@@ -100,11 +106,12 @@ class APIRequestContext(ChannelOwner):
         super().__init__(parent, type, guid, initializer)
         self._tracing: Tracing = from_channel(initializer["tracing"])
         self._close_reason: Optional[str] = None
+        self._timeout_settings = TimeoutSettings(None)
 
     async def dispose(self, reason: str = None) -> None:
         self._close_reason = reason
         try:
-            await self._channel.send("dispose", {"reason": reason})
+            await self._channel.send("dispose", None, {"reason": reason})
         except Error as e:
             if is_target_closed_error(e):
                 return
@@ -402,6 +409,7 @@ class APIRequestContext(ChannelOwner):
 
         response = await self._channel.send(
             "fetch",
+            self._timeout_settings.timeout,
             {
                 "url": url,
                 "params": object_to_array(params) if isinstance(params, dict) else None,
@@ -412,7 +420,6 @@ class APIRequestContext(ChannelOwner):
                 "jsonData": json_data,
                 "formData": form_data,
                 "multipartData": multipart_data,
-                "timeout": timeout,
                 "failOnStatusCode": failOnStatusCode,
                 "ignoreHTTPSErrors": ignoreHTTPSErrors,
                 "maxRedirects": maxRedirects,
@@ -422,9 +429,13 @@ class APIRequestContext(ChannelOwner):
         return APIResponse(self, response)
 
     async def storage_state(
-        self, path: Union[pathlib.Path, str] = None
+        self,
+        path: Union[pathlib.Path, str] = None,
+        indexedDB: bool = None,
     ) -> StorageState:
-        result = await self._channel.send_return_as_dict("storageState")
+        result = await self._channel.send_return_as_dict(
+            "storageState", None, {"indexedDB": indexedDB}
+        )
         if path:
             await async_writefile(path, json.dumps(result))
         return result
@@ -475,11 +486,15 @@ class APIResponse:
 
     async def body(self) -> bytes:
         try:
-            result = await self._request._channel.send_return_as_dict(
-                "fetchResponseBody",
-                {
-                    "fetchUid": self._fetch_uid,
-                },
+            result = await self._request._connection.wrap_api_call(
+                lambda: self._request._channel.send_return_as_dict(
+                    "fetchResponseBody",
+                    None,
+                    {
+                        "fetchUid": self._fetch_uid,
+                    },
+                ),
+                True,
             )
             if result is None:
                 raise Error("Response has been disposed")
@@ -500,6 +515,7 @@ class APIResponse:
     async def dispose(self) -> None:
         await self._request._channel.send(
             "disposeAPIResponse",
+            None,
             {
                 "fetchUid": self._fetch_uid,
             },
@@ -512,6 +528,7 @@ class APIResponse:
     async def _fetch_log(self) -> List[str]:
         return await self._request._channel.send(
             "fetchLog",
+            None,
             {
                 "fetchUid": self._fetch_uid,
             },
