@@ -107,7 +107,7 @@ class Locator:
         task: Callable[[ElementHandle, float], Awaitable[T]],
         timeout: float = None,
     ) -> T:
-        timeout = self._frame.page._timeout_settings.timeout(timeout)
+        timeout = self._frame._timeout(timeout)
         deadline = (monotonic_time() + timeout) if timeout else 0
         handle = await self.element_handle(timeout=timeout)
         if not handle:
@@ -336,6 +336,12 @@ class Locator:
     def content_frame(self) -> "FrameLocator":
         return FrameLocator(self._frame, self._selector)
 
+    def describe(self, description: str) -> "Locator":
+        return Locator(
+            self._frame,
+            f"{self._selector} >> internal:describe={json.dumps(description)}",
+        )
+
     def filter(
         self,
         hasText: Union[str, Pattern[str]] = None,
@@ -377,6 +383,7 @@ class Locator:
     async def blur(self, timeout: float = None) -> None:
         await self._frame._channel.send(
             "blur",
+            self._frame._timeout,
             {
                 "selector": self._selector,
                 "strict": True,
@@ -540,9 +547,10 @@ class Locator:
             ),
         )
 
-    async def aria_snapshot(self, timeout: float = None, ref: bool = None) -> str:
+    async def aria_snapshot(self, timeout: float = None) -> str:
         return await self._frame._channel.send(
             "ariaSnapshot",
+            self._frame._timeout,
             {
                 "selector": self._selector,
                 **locals_to_params(locals()),
@@ -711,17 +719,22 @@ class Locator:
             )
 
     async def _expect(
-        self, expression: str, options: FrameExpectOptions
+        self,
+        expression: str,
+        options: FrameExpectOptions,
+        title: str = None,
     ) -> FrameExpectResult:
         if "expectedValue" in options:
             options["expectedValue"] = serialize_argument(options["expectedValue"])
         result = await self._frame._channel.send_return_as_dict(
             "expect",
+            self._frame._timeout,
             {
                 "selector": self._selector,
                 "expression": expression,
                 **options,
             },
+            title=title,
         )
         if result.get("received"):
             result["received"] = parse_value(result["received"])
