@@ -131,7 +131,6 @@ class Request(ChannelOwner):
         self, parent: ChannelOwner, type: str, guid: str, initializer: Dict
     ) -> None:
         super().__init__(parent, type, guid, initializer)
-        self._channel.mark_as_internal_type()
         self._redirected_from: Optional["Request"] = from_nullable_channel(
             initializer.get("redirectedFrom")
         )
@@ -193,7 +192,10 @@ class Request(ChannelOwner):
         response = await self.response()
         if not response:
             raise Error("Unable to fetch sizes for failed request")
-        return await response._channel.send("sizes")
+        return await response._channel.send(
+            "sizes",
+            None,
+        )
 
     @property
     def post_data(self) -> Optional[str]:
@@ -227,7 +229,12 @@ class Request(ChannelOwner):
         return None
 
     async def response(self) -> Optional["Response"]:
-        return from_nullable_channel(await self._channel.send("response"))
+        return from_nullable_channel(
+            await self._channel.send(
+                "response",
+                None,
+            )
+        )
 
     @property
     def frame(self) -> "Frame":
@@ -292,7 +299,9 @@ class Request(ChannelOwner):
             return RawHeaders(serialize_headers(override))
         if not self._all_headers_future:
             self._all_headers_future = asyncio.Future()
-            headers = await self._channel.send("rawRequestHeaders")
+            headers = await self._channel.send(
+                "rawRequestHeaders", None, is_internal=True
+            )
             self._all_headers_future.set_result(RawHeaders(headers))
         return await self._all_headers_future
 
@@ -319,7 +328,6 @@ class Route(ChannelOwner):
         self, parent: ChannelOwner, type: str, guid: str, initializer: Dict
     ) -> None:
         super().__init__(parent, type, guid, initializer)
-        self._channel.mark_as_internal_type()
         self._handling_future: Optional[asyncio.Future["bool"]] = None
         self._context: "BrowserContext" = cast("BrowserContext", None)
         self._did_throw = False
@@ -350,6 +358,7 @@ class Route(ChannelOwner):
             lambda: self._race_with_page_close(
                 self._channel.send(
                     "abort",
+                    None,
                     {
                         "errorCode": errorCode,
                     },
@@ -435,7 +444,7 @@ class Route(ChannelOwner):
             headers["content-length"] = str(length)
         params["headers"] = serialize_headers(headers)
 
-        await self._race_with_page_close(self._channel.send("fulfill", params))
+        await self._race_with_page_close(self._channel.send("fulfill", None, params))
 
     async def _handle_route(self, callback: Callable) -> None:
         self._check_not_handled()
@@ -501,6 +510,7 @@ class Route(ChannelOwner):
         await self._race_with_page_close(
             self._channel.send(
                 "continue",
+                None,
                 {
                     "url": options.url,
                     "method": options.method,
@@ -520,7 +530,7 @@ class Route(ChannelOwner):
     async def _redirected_navigation_request(self, url: str) -> None:
         await self._handle_route(
             lambda: self._race_with_page_close(
-                self._channel.send("redirectNavigationRequest", {"url": url})
+                self._channel.send("redirectNavigationRequest", None, {"url": url})
             )
         )
 
@@ -530,7 +540,7 @@ class Route(ChannelOwner):
         setattr(
             fut,
             "__pw_stack__",
-            getattr(asyncio.current_task(self._loop), "__pw_stack__", inspect.stack()),
+            getattr(asyncio.current_task(self._loop), "__pw_stack__", inspect.stack(0)),
         )
         target_closed_future = self.request._target_closed_future()
         await asyncio.wait(
@@ -579,6 +589,7 @@ class ServerWebSocketRoute:
             self._ws._loop,
             self._ws._channel.send(
                 "closeServer",
+                None,
                 {
                     "code": code,
                     "reason": reason,
@@ -592,7 +603,7 @@ class ServerWebSocketRoute:
             _create_task_and_ignore_exception(
                 self._ws._loop,
                 self._ws._channel.send(
-                    "sendToServer", {"message": message, "isBase64": False}
+                    "sendToServer", None, {"message": message, "isBase64": False}
                 ),
             )
         else:
@@ -600,6 +611,7 @@ class ServerWebSocketRoute:
                 self._ws._loop,
                 self._ws._channel.send(
                     "sendToServer",
+                    None,
                     {"message": base64.b64encode(message).decode(), "isBase64": True},
                 ),
             )
@@ -610,7 +622,6 @@ class WebSocketRoute(ChannelOwner):
         self, parent: ChannelOwner, type: str, guid: str, initializer: Dict
     ) -> None:
         super().__init__(parent, type, guid, initializer)
-        self._channel.mark_as_internal_type()
         self._on_page_message: Optional[Callable[[Union[str, bytes]], Any]] = None
         self._on_page_close: Optional[Callable[[Optional[int], Optional[str]], Any]] = (
             None
@@ -636,7 +647,7 @@ class WebSocketRoute(ChannelOwner):
             )
         elif self._connected:
             _create_task_and_ignore_exception(
-                self._loop, self._channel.send("sendToServer", event)
+                self._loop, self._channel.send("sendToServer", None, event)
             )
 
     def _channel_message_from_server(self, event: Dict) -> None:
@@ -648,7 +659,7 @@ class WebSocketRoute(ChannelOwner):
             )
         else:
             _create_task_and_ignore_exception(
-                self._loop, self._channel.send("sendToPage", event)
+                self._loop, self._channel.send("sendToPage", None, event)
             )
 
     def _channel_close_page(self, event: Dict) -> None:
@@ -656,7 +667,7 @@ class WebSocketRoute(ChannelOwner):
             self._on_page_close(event["code"], event["reason"])
         else:
             _create_task_and_ignore_exception(
-                self._loop, self._channel.send("closeServer", event)
+                self._loop, self._channel.send("closeServer", None, event)
             )
 
     def _channel_close_server(self, event: Dict) -> None:
@@ -664,7 +675,7 @@ class WebSocketRoute(ChannelOwner):
             self._on_server_close(event["code"], event["reason"])
         else:
             _create_task_and_ignore_exception(
-                self._loop, self._channel.send("closePage", event)
+                self._loop, self._channel.send("closePage", None, event)
             )
 
     @property
@@ -674,7 +685,7 @@ class WebSocketRoute(ChannelOwner):
     async def close(self, code: int = None, reason: str = None) -> None:
         try:
             await self._channel.send(
-                "closePage", {"code": code, "reason": reason, "wasClean": True}
+                "closePage", None, {"code": code, "reason": reason, "wasClean": True}
             )
         except Exception:
             pass
@@ -683,7 +694,12 @@ class WebSocketRoute(ChannelOwner):
         if self._connected:
             raise Error("Already connected to the server")
         self._connected = True
-        asyncio.create_task(self._channel.send("connect"))
+        asyncio.create_task(
+            self._channel.send(
+                "connect",
+                None,
+            )
+        )
         return cast("WebSocketRoute", self._server)
 
     def send(self, message: Union[str, bytes]) -> None:
@@ -691,7 +707,7 @@ class WebSocketRoute(ChannelOwner):
             _create_task_and_ignore_exception(
                 self._loop,
                 self._channel.send(
-                    "sendToPage", {"message": message, "isBase64": False}
+                    "sendToPage", None, {"message": message, "isBase64": False}
                 ),
             )
         else:
@@ -699,6 +715,7 @@ class WebSocketRoute(ChannelOwner):
                 self._loop,
                 self._channel.send(
                     "sendToPage",
+                    None,
                     {
                         "message": base64.b64encode(message).decode(),
                         "isBase64": True,
@@ -716,7 +733,10 @@ class WebSocketRoute(ChannelOwner):
         if self._connected:
             return
         # Ensure that websocket is "open" and can send messages without an actual server connection.
-        await self._channel.send("ensureOpened")
+        await self._channel.send(
+            "ensureOpened",
+            None,
+        )
 
 
 class WebSocketRouteHandler:
@@ -768,7 +788,6 @@ class Response(ChannelOwner):
         self, parent: ChannelOwner, type: str, guid: str, initializer: Dict
     ) -> None:
         super().__init__(parent, type, guid, initializer)
-        self._channel.mark_as_internal_type()
         self._request: Request = from_channel(self._initializer["request"])
         timing = self._initializer["timing"]
         self._request._timing["startTime"] = timing["startTime"]
@@ -830,15 +849,27 @@ class Response(ChannelOwner):
     async def _actual_headers(self) -> "RawHeaders":
         if not self._raw_headers_future:
             self._raw_headers_future = asyncio.Future()
-            headers = cast(HeadersArray, await self._channel.send("rawResponseHeaders"))
+            headers = cast(
+                HeadersArray,
+                await self._channel.send(
+                    "rawResponseHeaders",
+                    None,
+                ),
+            )
             self._raw_headers_future.set_result(RawHeaders(headers))
         return await self._raw_headers_future
 
     async def server_addr(self) -> Optional[RemoteAddr]:
-        return await self._channel.send("serverAddr")
+        return await self._channel.send(
+            "serverAddr",
+            None,
+        )
 
     async def security_details(self) -> Optional[SecurityDetails]:
-        return await self._channel.send("securityDetails")
+        return await self._channel.send(
+            "securityDetails",
+            None,
+        )
 
     async def finished(self) -> None:
         async def on_finished() -> None:
@@ -857,7 +888,10 @@ class Response(ChannelOwner):
             await on_finished_task
 
     async def body(self) -> bytes:
-        binary = await self._channel.send("body")
+        binary = await self._channel.send(
+            "body",
+            None,
+        )
         return base64.b64decode(binary)
 
     async def text(self) -> str:
