@@ -60,6 +60,7 @@ from playwright._impl._greenlets import LocatorHandlerGreenlet
 from playwright._impl._har_router import HarRouter
 from playwright._impl._helper import (
     ColorScheme,
+    Contrast,
     DocumentLoadState,
     ForcedColors,
     HarMode,
@@ -226,6 +227,7 @@ class Page(ChannelOwner):
             ),
         )
         self._channel.on("video", lambda params: self._on_video(params))
+        self._channel.on("viewportSizeChanged", self._on_viewport_size_changed)
         self._channel.on(
             "webSocket",
             lambda params: self.emit(
@@ -285,7 +287,7 @@ class Page(ChannelOwner):
         route_handlers = self._routes.copy()
         for route_handler in route_handlers:
             # If the page was closed we stall all requests right away.
-            if self._close_was_called or self.context._close_was_called:
+            if self._close_was_called or self.context._closing_or_closed:
                 return
             if not route_handler.matches(route.request.url):
                 continue
@@ -362,6 +364,9 @@ class Page(ChannelOwner):
         artifact = from_channel(params["artifact"])
         self._force_video()._artifact_ready(artifact)
 
+    def _on_viewport_size_changed(self, params: Any) -> None:
+        self._viewport_size = params["viewportSize"]
+
     @property
     def context(self) -> "BrowserContext":
         return self._browser_context
@@ -396,13 +401,9 @@ class Page(ChannelOwner):
 
     def set_default_navigation_timeout(self, timeout: float) -> None:
         self._timeout_settings.set_default_navigation_timeout(timeout)
-        self._channel.send_no_reply(
-            "setDefaultNavigationTimeoutNoReply", dict(timeout=timeout)
-        )
 
     def set_default_timeout(self, timeout: float) -> None:
         self._timeout_settings.set_default_timeout(timeout)
-        self._channel.send_no_reply("setDefaultTimeoutNoReply", dict(timeout=timeout))
 
     async def query_selector(
         self,
@@ -518,12 +519,16 @@ class Page(ChannelOwner):
             )
         self._bindings[name] = callback
         await self._channel.send(
-            "exposeBinding", dict(name=name, needsHandle=handle or False)
+            "exposeBinding",
+            None,
+            dict(name=name, needsHandle=handle or False),
         )
 
     async def set_extra_http_headers(self, headers: Dict[str, str]) -> None:
         await self._channel.send(
-            "setExtraHTTPHeaders", dict(headers=serialize_headers(headers))
+            "setExtraHTTPHeaders",
+            None,
+            dict(headers=serialize_headers(headers)),
         )
 
     @property
@@ -556,7 +561,11 @@ class Page(ChannelOwner):
         waitUntil: DocumentLoadState = None,
     ) -> Optional[Response]:
         return from_nullable_channel(
-            await self._channel.send("reload", locals_to_params(locals()))
+            await self._channel.send(
+                "reload",
+                self._timeout_settings.navigation_timeout,
+                locals_to_params(locals()),
+            )
         )
 
     async def wait_for_load_state(
@@ -587,7 +596,11 @@ class Page(ChannelOwner):
         waitUntil: DocumentLoadState = None,
     ) -> Optional[Response]:
         return from_nullable_channel(
-            await self._channel.send("goBack", locals_to_params(locals()))
+            await self._channel.send(
+                "goBack",
+                self._timeout_settings.navigation_timeout,
+                locals_to_params(locals()),
+            )
         )
 
     async def go_forward(
@@ -596,11 +609,15 @@ class Page(ChannelOwner):
         waitUntil: DocumentLoadState = None,
     ) -> Optional[Response]:
         return from_nullable_channel(
-            await self._channel.send("goForward", locals_to_params(locals()))
+            await self._channel.send(
+                "goForward",
+                self._timeout_settings.navigation_timeout,
+                locals_to_params(locals()),
+            )
         )
 
     async def request_gc(self) -> None:
-        await self._channel.send("requestGC")
+        await self._channel.send("requestGC", None)
 
     async def emulate_media(
         self,
@@ -608,6 +625,7 @@ class Page(ChannelOwner):
         colorScheme: ColorScheme = None,
         reducedMotion: ReducedMotion = None,
         forcedColors: ForcedColors = None,
+        contrast: Contrast = None,
     ) -> None:
         params = locals_to_params(locals())
         if "media" in params:
@@ -624,18 +642,26 @@ class Page(ChannelOwner):
             params["forcedColors"] = (
                 "no-override" if params["forcedColors"] == "null" else forcedColors
             )
-        await self._channel.send("emulateMedia", params)
+        if "contrast" in params:
+            params["contrast"] = (
+                "no-override" if params["contrast"] == "null" else contrast
+            )
+        await self._channel.send("emulateMedia", None, params)
 
     async def set_viewport_size(self, viewportSize: ViewportSize) -> None:
         self._viewport_size = viewportSize
-        await self._channel.send("setViewportSize", locals_to_params(locals()))
+        await self._channel.send(
+            "setViewportSize",
+            None,
+            locals_to_params(locals()),
+        )
 
     @property
     def viewport_size(self) -> Optional[ViewportSize]:
         return self._viewport_size
 
     async def bring_to_front(self) -> None:
-        await self._channel.send("bringToFront")
+        await self._channel.send("bringToFront", None)
 
     async def add_init_script(
         self, script: str = None, path: Union[str, Path] = None
@@ -646,7 +672,7 @@ class Page(ChannelOwner):
             )
         if not isinstance(script, str):
             raise Error("Either path or script parameter must be specified")
-        await self._channel.send("addInitScript", dict(source=script))
+        await self._channel.send("addInitScript", None, dict(source=script))
 
     async def route(
         self, url: URLMatch, handler: RouteHandlerCallback, times: int = None
@@ -744,7 +770,9 @@ class Page(ChannelOwner):
     async def _update_interception_patterns(self) -> None:
         patterns = RouteHandler.prepare_interception_patterns(self._routes)
         await self._channel.send(
-            "setNetworkInterceptionPatterns", {"patterns": patterns}
+            "setNetworkInterceptionPatterns",
+            None,
+            {"patterns": patterns},
         )
 
     async def _update_web_socket_interception_patterns(self) -> None:
@@ -752,7 +780,9 @@ class Page(ChannelOwner):
             self._web_socket_routes
         )
         await self._channel.send(
-            "setWebSocketInterceptionPatterns", {"patterns": patterns}
+            "setWebSocketInterceptionPatterns",
+            None,
+            {"patterns": patterns},
         )
 
     async def screenshot(
@@ -786,7 +816,9 @@ class Page(ChannelOwner):
                     params["mask"],
                 )
             )
-        encoded_binary = await self._channel.send("screenshot", params)
+        encoded_binary = await self._channel.send(
+            "screenshot", self._timeout_settings.timeout, params
+        )
         decoded_binary = base64.b64decode(encoded_binary)
         if path:
             make_dirs_for_file(path)
@@ -800,7 +832,7 @@ class Page(ChannelOwner):
         self._close_reason = reason
         self._close_was_called = True
         try:
-            await self._channel.send("close", locals_to_params(locals()))
+            await self._channel.send("close", None, locals_to_params(locals()))
             if self._owned_context:
                 await self._owned_context.close()
         except Exception as e:
@@ -1099,7 +1131,9 @@ class Page(ChannelOwner):
         try:
             await asyncio.wait(
                 [
-                    asyncio.create_task(self._browser_context._channel.send("pause")),
+                    asyncio.create_task(
+                        self._browser_context._channel.send("pause", None)
+                    ),
                     self._closed_or_crashed_future,
                 ],
                 return_when=asyncio.FIRST_COMPLETED,
@@ -1131,7 +1165,7 @@ class Page(ChannelOwner):
         params = locals_to_params(locals())
         if "path" in params:
             del params["path"]
-        encoded_binary = await self._channel.send("pdf", params)
+        encoded_binary = await self._channel.send("pdf", None, params)
         decoded_binary = base64.b64decode(encoded_binary)
         if path:
             make_dirs_for_file(path)
@@ -1341,6 +1375,7 @@ class Page(ChannelOwner):
             return
         uid = await self._channel.send(
             "registerLocatorHandler",
+            None,
             {
                 "selector": locator._selector,
                 "noWaitAfter": noWaitAfter,
@@ -1381,7 +1416,9 @@ class Page(ChannelOwner):
             try:
                 await self._connection.wrap_api_call(
                     lambda: self._channel.send(
-                        "resolveLocatorHandlerNoReply", {"uid": uid, "remove": remove}
+                        "resolveLocatorHandlerNoReply",
+                        None,
+                        {"uid": uid, "remove": remove},
                     ),
                     is_internal=True,
                 )
@@ -1392,7 +1429,11 @@ class Page(ChannelOwner):
         for uid, data in self._locator_handlers.copy().items():
             if data.locator._equals(locator):
                 del self._locator_handlers[uid]
-                self._channel.send_no_reply("unregisterLocatorHandler", {"uid": uid})
+                self._channel.send_no_reply(
+                    "unregisterLocatorHandler",
+                    None,
+                    {"uid": uid},
+                )
 
 
 class Worker(ChannelOwner):
@@ -1424,6 +1465,7 @@ class Worker(ChannelOwner):
         return parse_result(
             await self._channel.send(
                 "evaluateExpression",
+                None,
                 dict(
                     expression=expression,
                     arg=serialize_argument(arg),
@@ -1437,6 +1479,7 @@ class Worker(ChannelOwner):
         return from_channel(
             await self._channel.send(
                 "evaluateExpressionHandle",
+                None,
                 dict(
                     expression=expression,
                     arg=serialize_argument(arg),
@@ -1462,12 +1505,14 @@ class BindingCall(ChannelOwner):
                 result = func(source, *func_args)
             if inspect.iscoroutine(result):
                 result = await result
-            await self._channel.send("resolve", dict(result=serialize_argument(result)))
+            await self._channel.send(
+                "resolve", None, dict(result=serialize_argument(result))
+            )
         except Exception as e:
             tb = sys.exc_info()[2]
             asyncio.create_task(
                 self._channel.send(
-                    "reject", dict(error=dict(error=serialize_error(e, tb)))
+                    "reject", None, dict(error=dict(error=serialize_error(e, tb)))
                 )
             )
 
