@@ -56,8 +56,8 @@ class Locator {
     if (this._frame._platform.inspectCustom)
       this[this._frame._platform.inspectCustom] = () => this._inspect();
   }
-  async _withElement(task, timeout) {
-    timeout = this._frame.page()._timeoutSettings.timeout({ timeout });
+  async _withElement(task, options) {
+    const timeout = this._frame._timeout({ timeout: options.timeout });
     const deadline = timeout ? (0, import_time.monotonicTime)() + timeout : 0;
     return await this._frame._wrapApiCall(async () => {
       const result = await this._frame._channel.waitForSelector({ selector: this._selector, strict: true, state: "attached", timeout });
@@ -69,7 +69,7 @@ class Locator {
       } finally {
         await handle.dispose();
       }
-    });
+    }, { title: options.title, internal: options.internal });
   }
   _equals(locator) {
     return this._frame === locator._frame && this._selector === locator._selector;
@@ -78,7 +78,7 @@ class Locator {
     return this._frame.page();
   }
   async boundingBox(options) {
-    return await this._withElement((h) => h.boundingBox(), options?.timeout);
+    return await this._withElement((h) => h.boundingBox(), { title: "Bounding box", timeout: options?.timeout });
   }
   async check(options = {}) {
     return await this._frame.check(this._selector, { strict: true, ...options });
@@ -87,7 +87,7 @@ class Locator {
     return await this._frame.click(this._selector, { strict: true, ...options });
   }
   async dblclick(options = {}) {
-    return await this._frame.dblclick(this._selector, { strict: true, ...options });
+    await this._frame.dblclick(this._selector, { strict: true, ...options });
   }
   async dispatchEvent(type, eventInit = {}, options) {
     return await this._frame.dispatchEvent(this._selector, type, eventInit, { strict: true, ...options });
@@ -99,13 +99,13 @@ class Locator {
     });
   }
   async evaluate(pageFunction, arg, options) {
-    return await this._withElement((h) => h.evaluate(pageFunction, arg), options?.timeout);
+    return await this._withElement((h) => h.evaluate(pageFunction, arg), { title: "Evaluate", timeout: options?.timeout });
   }
   async evaluateAll(pageFunction, arg) {
     return await this._frame.$$eval(this._selector, pageFunction, arg);
   }
   async evaluateHandle(pageFunction, arg, options) {
-    return await this._withElement((h) => h.evaluateHandle(pageFunction, arg), options?.timeout);
+    return await this._withElement((h) => h.evaluateHandle(pageFunction, arg), { title: "Evaluate", timeout: options?.timeout });
   }
   async fill(value, options = {}) {
     return await this._frame.fill(this._selector, value, { strict: true, ...options });
@@ -162,6 +162,9 @@ class Locator {
   contentFrame() {
     return new FrameLocator(this._frame, this._selector);
   }
+  describe(description) {
+    return new Locator(this._frame, this._selector + " >> internal:describe=" + JSON.stringify(description));
+  }
   first() {
     return new Locator(this._frame, this._selector + " >> nth=0");
   }
@@ -185,13 +188,13 @@ class Locator {
     return await this._frame.focus(this._selector, { strict: true, ...options });
   }
   async blur(options) {
-    await this._frame._channel.blur({ selector: this._selector, strict: true, ...options });
+    await this._frame._channel.blur({ selector: this._selector, strict: true, ...options, timeout: this._frame._timeout(options) });
   }
   async count() {
     return await this._frame._queryCount(this._selector);
   }
   async _generateLocatorString() {
-    return await this._withElement((h) => h._generateLocatorString());
+    return await this._withElement((h) => h._generateLocatorString(), { title: "Generate locator string", internal: true });
   }
   async getAttribute(name, options) {
     return await this._frame.getAttribute(this._selector, name, { strict: true, ...options });
@@ -231,20 +234,20 @@ class Locator {
   }
   async screenshot(options = {}) {
     const mask = options.mask;
-    return await this._withElement((h, timeout) => h.screenshot({ ...options, mask, timeout }), options.timeout);
+    return await this._withElement((h, timeout) => h.screenshot({ ...options, mask, timeout }), { title: "Screenshot", timeout: options.timeout });
   }
   async ariaSnapshot(options) {
-    const result = await this._frame._channel.ariaSnapshot({ ...options, mode: options?._mode, selector: this._selector });
+    const result = await this._frame._channel.ariaSnapshot({ ...options, selector: this._selector, timeout: this._frame._timeout(options) });
     return result.snapshot;
   }
   async scrollIntoViewIfNeeded(options = {}) {
-    return await this._withElement((h, timeout) => h.scrollIntoViewIfNeeded({ ...options, timeout }), options.timeout);
+    return await this._withElement((h, timeout) => h.scrollIntoViewIfNeeded({ ...options, timeout }), { title: "Scroll into view", timeout: options.timeout });
   }
   async selectOption(values, options = {}) {
     return await this._frame.selectOption(this._selector, values, { strict: true, ...options });
   }
   async selectText(options = {}) {
-    return await this._withElement((h, timeout) => h.selectText({ ...options, timeout }), options.timeout);
+    return await this._withElement((h, timeout) => h.selectText({ ...options, timeout }), { title: "Select text", timeout: options.timeout });
   }
   async setChecked(checked, options) {
     if (checked)
@@ -280,7 +283,7 @@ class Locator {
     return await this._frame.$$eval(this._selector, (ee) => ee.map((e) => e.textContent || ""));
   }
   async waitFor(options) {
-    await this._frame._channel.waitForSelector({ selector: this._selector, strict: true, omitReturnValue: true, ...options });
+    await this._frame._channel.waitForSelector({ selector: this._selector, strict: true, omitReturnValue: true, ...options, timeout: this._frame._timeout(options) });
   }
   async _expect(expression, options) {
     const params = { selector: this._selector, expression, ...options, isNot: !!options.isNot };
