@@ -92,8 +92,8 @@ class JSCoverage {
       this._scriptSources.set(event.scriptId, response.scriptSource);
   }
   async stop() {
-    (0, import_utils.assert)(this._enabled, "JSCoverage is not enabled");
-    this._enabled = false;
+    if (!this._enabled)
+      return { entries: [] };
     const [profileResponse] = await Promise.all([
       this._client.send("Profiler.takePreciseCoverage"),
       this._client.send("Profiler.stopPreciseCoverage"),
@@ -101,6 +101,7 @@ class JSCoverage {
       this._client.send("Debugger.disable")
     ]);
     import_eventsHelper.eventsHelper.removeEventListeners(this._eventListeners);
+    this._enabled = false;
     const coverage = { entries: [] };
     for (const entry of profileResponse.result) {
       if (!this._scriptIds.has(entry.scriptId))
@@ -159,14 +160,15 @@ class CSSCoverage {
     }
   }
   async stop() {
-    (0, import_utils.assert)(this._enabled, "CSSCoverage is not enabled");
-    this._enabled = false;
+    if (!this._enabled)
+      return { entries: [] };
     const ruleTrackingResponse = await this._client.send("CSS.stopRuleUsageTracking");
     await Promise.all([
       this._client.send("CSS.disable"),
       this._client.send("DOM.disable")
     ]);
     import_eventsHelper.eventsHelper.removeEventListeners(this._eventListeners);
+    this._enabled = false;
     const styleSheetIdToCoverage = /* @__PURE__ */ new Map();
     for (const entry of ruleTrackingResponse.ruleUsage) {
       let ranges = styleSheetIdToCoverage.get(entry.styleSheetId);
