@@ -44,7 +44,7 @@ var import_progress = require("./progress");
 var import_utils = require("../utils");
 var import_fileUploadUtils = require("./fileUploadUtils");
 var import_protocolError = require("./protocolError");
-var injectedScriptSource = __toESM(require("../generated/injectedScriptSource"));
+var rawInjectedScriptSource = __toESM(require("../generated/injectedScriptSource"));
 class NonRecoverableDOMError extends Error {
 }
 function isNonRecoverableDOMError(error) {
@@ -58,7 +58,7 @@ class FrameExecutionContext extends js.ExecutionContext {
   }
   adoptIfNeeded(handle) {
     if (handle instanceof ElementHandle && handle._context !== this)
-      return this.frame._page._delegate.adoptElementHandle(handle, this);
+      return this.frame._page.delegate.adoptElementHandle(handle, this);
     return null;
   }
   async evaluate(pageFunction, arg) {
@@ -75,25 +75,25 @@ class FrameExecutionContext extends js.ExecutionContext {
   }
   injectedScript() {
     if (!this._injectedScriptPromise) {
-      const custom = [];
-      const selectorsRegistry = this.frame._page.context().selectors();
+      const customEngines = [];
+      const selectorsRegistry = this.frame._page.browserContext.selectors();
       for (const [name, { source: source2 }] of selectorsRegistry._engines)
-        custom.push(`{ name: '${name}', engine: (${source2}) }`);
+        customEngines.push({ name, source: `(${source2})` });
       const sdkLanguage = this.frame.attribution.playwright.options.sdkLanguage;
+      const options = {
+        isUnderTest: (0, import_utils.isUnderTest)(),
+        sdkLanguage,
+        testIdAttributeName: selectorsRegistry.testIdAttributeName(),
+        stableRafCount: this.frame._page.delegate.rafCountForStablePosition(),
+        browserName: this.frame._page.browserContext._browser.options.name,
+        inputFileRoleTextbox: process.env.PLAYWRIGHT_INPUT_FILE_TEXTBOX ? true : false,
+        customEngines
+      };
       const source = `
         (() => {
         const module = {};
-        ${injectedScriptSource.source}
-        return new (module.exports.InjectedScript())(
-          globalThis,
-          ${(0, import_utils.isUnderTest)()},
-          "${sdkLanguage}",
-          ${JSON.stringify(selectorsRegistry.testIdAttributeName())},
-          ${this.frame._page._delegate.rafCountForStablePosition()},
-          "${this.frame._page._browserContext._browser.options.name}",
-          ${process.env.PLAYWRIGHT_INPUT_FILE_TEXTBOX ? "true" : "false"},
-          [${custom.join(",\n")}]
-        );
+        ${rawInjectedScriptSource.source}
+        return new (module.exports.InjectedScript())(globalThis, ${JSON.stringify(options)});
         })();
       `;
       this._injectedScriptPromise = this.rawEvaluateHandle(source).then((handle) => {
@@ -141,14 +141,14 @@ class ElementHandle extends js.JSHandle {
     }
   }
   async ownerFrame() {
-    const frameId = await this._page._delegate.getOwnerFrame(this);
+    const frameId = await this._page.delegate.getOwnerFrame(this);
     if (!frameId)
       return null;
-    const frame = this._page._frameManager.frame(frameId);
+    const frame = this._page.frameManager.frame(frameId);
     if (frame)
       return frame;
-    for (const page of this._page._browserContext.pages()) {
-      const frame2 = page._frameManager.frame(frameId);
+    for (const page of this._page.browserContext.pages()) {
+      const frame2 = page.frameManager.frame(frameId);
       if (frame2)
         return frame2;
     }
@@ -161,36 +161,57 @@ class ElementHandle extends js.JSHandle {
     const isFrameElement = throwRetargetableDOMError(await this.isIframeElement());
     if (!isFrameElement)
       return null;
-    return this._page._delegate.getContentFrame(this);
+    return this._page.delegate.getContentFrame(this);
   }
   async generateLocatorString() {
+    const selectors = await this._generateSelectorString();
+    if (!selectors.length)
+      return;
+    return (0, import_utils.asLocator)("javascript", selectors.reverse().join(" >> internal:control=enter-frame >> "));
+  }
+  async _generateSelectorString() {
     const selector = await this.evaluateInUtility(async ([injected, node]) => {
       return injected.generateSelectorSimple(node);
     }, {});
     if (selector === "error:notconnected")
-      return;
-    return (0, import_utils.asLocator)("javascript", selector);
+      return [];
+    let frame = this._frame;
+    const result = [selector];
+    while (frame?.parentFrame()) {
+      const frameElement = await frame.frameElement();
+      if (frameElement) {
+        const selector2 = await frameElement.evaluateInUtility(async ([injected, node]) => {
+          return injected.generateSelectorSimple(node);
+        }, {});
+        frameElement.dispose();
+        if (selector2 === "error:notconnected")
+          return [];
+        result.push(selector2);
+      }
+      frame = frame.parentFrame();
+    }
+    return result;
   }
   async getAttribute(metadata, name) {
-    return this._frame.getAttribute(metadata, ":scope", name, {}, this);
+    return this._frame.getAttribute(metadata, ":scope", name, { timeout: 0 }, this);
   }
   async inputValue(metadata) {
-    return this._frame.inputValue(metadata, ":scope", {}, this);
+    return this._frame.inputValue(metadata, ":scope", { timeout: 0 }, this);
   }
   async textContent(metadata) {
-    return this._frame.textContent(metadata, ":scope", {}, this);
+    return this._frame.textContent(metadata, ":scope", { timeout: 0 }, this);
   }
   async innerText(metadata) {
-    return this._frame.innerText(metadata, ":scope", {}, this);
+    return this._frame.innerText(metadata, ":scope", { timeout: 0 }, this);
   }
   async innerHTML(metadata) {
-    return this._frame.innerHTML(metadata, ":scope", {}, this);
+    return this._frame.innerHTML(metadata, ":scope", { timeout: 0 }, this);
   }
   async dispatchEvent(metadata, type, eventInit = {}) {
-    return this._frame.dispatchEvent(metadata, ":scope", type, eventInit, {}, this);
+    return this._frame.dispatchEvent(metadata, ":scope", type, eventInit, { timeout: 0 }, this);
   }
   async _scrollRectIntoViewIfNeeded(rect) {
-    return await this._page._delegate.scrollRectIntoViewIfNeeded(this, rect);
+    return await this._page.delegate.scrollRectIntoViewIfNeeded(this, rect);
   }
   async _waitAndScrollIntoViewIfNeeded(progress, waitForVisible) {
     const result = await this._retryAction(progress, "scroll into view", async () => {
@@ -204,7 +225,7 @@ class ElementHandle extends js.JSHandle {
     }, {});
     assertDone(throwRetargetableDOMError(result));
   }
-  async scrollIntoViewIfNeeded(metadata, options = {}) {
+  async scrollIntoViewIfNeeded(metadata, options) {
     const controller = new import_progress.ProgressController(metadata, this);
     return controller.run(
       (progress) => this._waitAndScrollIntoViewIfNeeded(
@@ -212,7 +233,7 @@ class ElementHandle extends js.JSHandle {
         false
         /* waitForVisible */
       ),
-      this._page._timeoutSettings.timeout(options)
+      options.timeout
     );
   }
   async _clickablePoint() {
@@ -232,7 +253,7 @@ class ElementHandle extends js.JSHandle {
       return Math.abs(area);
     };
     const [quads, metrics] = await Promise.all([
-      this._page._delegate.getContentQuads(this),
+      this._page.delegate.getContentQuads(this),
       this._page.mainFrame()._utilityContext().then((utility) => utility.evaluate(() => ({ width: innerWidth, height: innerHeight })))
     ]);
     if (quads === "error:notconnected")
@@ -242,7 +263,7 @@ class ElementHandle extends js.JSHandle {
     const filtered = quads.map((quad) => intersectQuadWithViewport(quad)).filter((quad) => computeQuadArea(quad) > 0.99);
     if (!filtered.length)
       return "error:notinviewport";
-    if (this._page._browserContext._browser.options.name === "firefox") {
+    if (this._page.browserContext._browser.options.name === "firefox") {
       for (const quad of filtered) {
         const integerPoint = findIntegerPointInsideQuad(quad);
         if (integerPoint)
@@ -284,7 +305,7 @@ class ElementHandle extends js.JSHandle {
       }
       if (!options.skipActionPreChecks && !options.force)
         await this._frame._page.performActionPreChecks(progress);
-      const result = await action(retry);
+      const result = await action();
       ++retry;
       if (result === "error:notvisible") {
         if (options.force)
@@ -316,15 +337,21 @@ class ElementHandle extends js.JSHandle {
   }
   async _retryPointerAction(progress, actionName, waitForEnabled, action, options) {
     const skipActionPreChecks = actionName === "move and up";
-    return await this._retryAction(progress, actionName, async (retry) => {
-      const scrollOptions = [
-        void 0,
-        { block: "end", inline: "end" },
-        { block: "center", inline: "center" },
-        { block: "start", inline: "start" }
-      ];
-      const forceScrollOptions = scrollOptions[retry % scrollOptions.length];
-      return await this._performPointerAction(progress, actionName, waitForEnabled, action, forceScrollOptions, options);
+    const scrollOptions = [
+      void 0,
+      { block: "end", inline: "end" },
+      { block: "center", inline: "center" },
+      { block: "start", inline: "start" }
+    ];
+    let scrollOptionIndex = 0;
+    return await this._retryAction(progress, actionName, async () => {
+      const forceScrollOptions = scrollOptions[scrollOptionIndex % scrollOptions.length];
+      const result = await this._performPointerAction(progress, actionName, waitForEnabled, action, forceScrollOptions, options);
+      if (typeof result === "object" && "hasPositionStickyOrFixed" in result && result.hasPositionStickyOrFixed)
+        ++scrollOptionIndex;
+      else
+        scrollOptionIndex = 0;
+      return result;
     }, { ...options, skipActionPreChecks });
   }
   async _performPointerAction(progress, actionName, waitForEnabled, action, forceScrollOptions, options) {
@@ -388,7 +415,7 @@ class ElementHandle extends js.JSHandle {
         const error = handle.rawValue();
         if (error === "error:notconnected")
           return error;
-        return { hitTargetDescription: error };
+        return JSON.parse(error);
       }
       hitTargetInterceptionHandle = handle;
       progress.cleanupWhenAborted(() => {
@@ -397,7 +424,7 @@ class ElementHandle extends js.JSHandle {
         hitTargetInterceptionHandle.dispose();
       });
     }
-    const actionResult = await this._page._frameManager.waitForSignalsCreatedBy(progress, options.waitAfter === true, async () => {
+    const actionResult = await this._page.frameManager.waitForSignalsCreatedBy(progress, options.waitAfter === true, async () => {
       if (options.__testHookBeforePointerAction)
         await options.__testHookBeforePointerAction();
       progress.throwIfAborted();
@@ -445,18 +472,18 @@ class ElementHandle extends js.JSHandle {
       await this._markAsTargetElement(metadata);
       const result = await this._hover(progress, options);
       return assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
   _hover(progress, options) {
     return this._retryPointerAction(progress, "hover", false, (point) => this._page.mouse.move(point.x, point.y), { ...options, waitAfter: "disabled" });
   }
-  async click(metadata, options = {}) {
+  async click(metadata, options) {
     const controller = new import_progress.ProgressController(metadata, this);
     return controller.run(async (progress) => {
       await this._markAsTargetElement(metadata);
       const result = await this._click(progress, { ...options, waitAfter: !options.noWaitAfter });
       return assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
   _click(progress, options) {
     return this._retryPointerAction(progress, "click", true, (point) => this._page.mouse.click(point.x, point.y, options), options);
@@ -467,18 +494,18 @@ class ElementHandle extends js.JSHandle {
       await this._markAsTargetElement(metadata);
       const result = await this._dblclick(progress, options);
       return assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
   _dblclick(progress, options) {
     return this._retryPointerAction(progress, "dblclick", true, (point) => this._page.mouse.dblclick(point.x, point.y, options), { ...options, waitAfter: "disabled" });
   }
-  async tap(metadata, options = {}) {
+  async tap(metadata, options) {
     const controller = new import_progress.ProgressController(metadata, this);
     return controller.run(async (progress) => {
       await this._markAsTargetElement(metadata);
       const result = await this._tap(progress, options);
       return assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
   _tap(progress, options) {
     return this._retryPointerAction(progress, "tap", true, (point) => this._page.touchscreen.tap(point.x, point.y), { ...options, waitAfter: "disabled" });
@@ -489,7 +516,7 @@ class ElementHandle extends js.JSHandle {
       await this._markAsTargetElement(metadata);
       const result = await this._selectOption(progress, elements, values, options);
       return throwRetargetableDOMError(result);
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
   async _selectOption(progress, elements, values, options) {
     let resultingOptions = [];
@@ -517,13 +544,13 @@ class ElementHandle extends js.JSHandle {
       return result;
     return resultingOptions;
   }
-  async fill(metadata, value, options = {}) {
+  async fill(metadata, value, options) {
     const controller = new import_progress.ProgressController(metadata, this);
     return controller.run(async (progress) => {
       await this._markAsTargetElement(metadata);
       const result = await this._fill(progress, value, options);
       assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
   async _fill(progress, value, options) {
     progress.log(`  fill("${value}")`);
@@ -551,7 +578,7 @@ class ElementHandle extends js.JSHandle {
       }
     }, options);
   }
-  async selectText(metadata, options = {}) {
+  async selectText(metadata, options) {
     const controller = new import_progress.ProgressController(metadata, this);
     return controller.run(async (progress) => {
       const result = await this._retryAction(progress, "selectText", async () => {
@@ -567,7 +594,7 @@ class ElementHandle extends js.JSHandle {
         }, { force: options.force });
       }, options);
       assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
   async setInputFiles(metadata, params) {
     const inputFileItems = await (0, import_fileUploadUtils.prepareFilesForUpload)(this._frame, params);
@@ -576,7 +603,7 @@ class ElementHandle extends js.JSHandle {
       await this._markAsTargetElement(metadata);
       const result = await this._setInputFiles(progress, inputFileItems);
       return assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(params));
+    }, params.timeout);
   }
   async _setInputFiles(progress, items) {
     const { filePayloads, localPaths, localDirectory } = items;
@@ -608,7 +635,7 @@ class ElementHandle extends js.JSHandle {
         node.addEventListener("input", fulfill, { once: true });
       })).catch(() => {
       }) : Promise.resolve();
-      await this._page._delegate.setInputFilePaths(retargeted, localPathsOrDirectory);
+      await this._page.delegate.setInputFilePaths(retargeted, localPathsOrDirectory);
       await waitForInputEvent;
     } else {
       await retargeted.evaluateInUtility(([injected, node, files]) => injected.setInputFiles(node, files), filePayloads);
@@ -637,7 +664,7 @@ class ElementHandle extends js.JSHandle {
       await this._markAsTargetElement(metadata);
       const result = await this._type(progress, text, options);
       return assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
   async _type(progress, text, options) {
     progress.log(`elementHandle.type("${text}")`);
@@ -659,12 +686,12 @@ class ElementHandle extends js.JSHandle {
       await this._markAsTargetElement(metadata);
       const result = await this._press(progress, key, options);
       return assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
   async _press(progress, key, options) {
     progress.log(`elementHandle.press("${key}")`);
     await this.instrumentation.onBeforeInputAction(this, progress.metadata);
-    return this._page._frameManager.waitForSignalsCreatedBy(progress, !options.noWaitAfter, async () => {
+    return this._page.frameManager.waitForSignalsCreatedBy(progress, !options.noWaitAfter, async () => {
       const result = await this._focus(
         progress,
         true
@@ -682,14 +709,14 @@ class ElementHandle extends js.JSHandle {
     return controller.run(async (progress) => {
       const result = await this._setChecked(progress, true, options);
       return assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
   async uncheck(metadata, options) {
     const controller = new import_progress.ProgressController(metadata, this);
     return controller.run(async (progress) => {
       const result = await this._setChecked(progress, false, options);
       return assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
   async _setChecked(progress, state, options) {
     const isChecked = async () => {
@@ -711,16 +738,16 @@ class ElementHandle extends js.JSHandle {
     return "done";
   }
   async boundingBox() {
-    return this._page._delegate.getBoundingBox(this);
+    return this._page.delegate.getBoundingBox(this);
   }
   async ariaSnapshot(options) {
     return await this.evaluateInUtility(([injected, element, options2]) => injected.ariaSnapshot(element, options2), options);
   }
-  async screenshot(metadata, options = {}) {
+  async screenshot(metadata, options) {
     const controller = new import_progress.ProgressController(metadata, this);
     return controller.run(
-      (progress) => this._page._screenshotter.screenshotElement(progress, this, options),
-      this._page._timeoutSettings.timeout(options)
+      (progress) => this._page.screenshotter.screenshotElement(progress, this, options),
+      options.timeout
     );
   }
   async querySelector(selector, options) {
@@ -742,18 +769,18 @@ class ElementHandle extends js.JSHandle {
     return this._frame.isHidden(metadata, ":scope", {}, this);
   }
   async isEnabled(metadata) {
-    return this._frame.isEnabled(metadata, ":scope", {}, this);
+    return this._frame.isEnabled(metadata, ":scope", { timeout: 0 }, this);
   }
   async isDisabled(metadata) {
-    return this._frame.isDisabled(metadata, ":scope", {}, this);
+    return this._frame.isDisabled(metadata, ":scope", { timeout: 0 }, this);
   }
   async isEditable(metadata) {
-    return this._frame.isEditable(metadata, ":scope", {}, this);
+    return this._frame.isEditable(metadata, ":scope", { timeout: 0 }, this);
   }
   async isChecked(metadata) {
-    return this._frame.isChecked(metadata, ":scope", {}, this);
+    return this._frame.isChecked(metadata, ":scope", { timeout: 0 }, this);
   }
-  async waitForElementState(metadata, state, options = {}) {
+  async waitForElementState(metadata, state, options) {
     const controller = new import_progress.ProgressController(metadata, this);
     return controller.run(async (progress) => {
       const actionName = `wait for ${state}`;
@@ -763,14 +790,14 @@ class ElementHandle extends js.JSHandle {
         }, state);
       }, {});
       assertDone(throwRetargetableDOMError(result));
-    }, this._page._timeoutSettings.timeout(options));
+    }, options.timeout);
   }
-  async waitForSelector(metadata, selector, options = {}) {
+  async waitForSelector(metadata, selector, options) {
     return this._frame.waitForSelector(metadata, selector, options, this);
   }
   async _adoptTo(context) {
     if (this._context !== context) {
-      const adopted = await this._page._delegate.adoptElementHandle(this, context);
+      const adopted = await this._page.delegate.adoptElementHandle(this, context);
       this.dispose();
       return adopted;
     }
